<?php

/**
 * @copyright Copyright (c) Metaways Infosystems GmbH, 2012
 * @license LGPLv3, http://www.arcavias.com/en/license
 * @package Client
 * @subpackage Html
 */


/**
 * Default implementation of catalog list section HTML clients.
 *
 * @package Client
 * @subpackage Html
 */
class Client_Html_Catalog_List_Default
	extends Client_Html_Catalog_Abstract
{
	/** client/html/catalog/list/default/subparts
	 * List of HTML sub-clients rendered within the catalog list section
	 *
	 * The output of the frontend is composed of the code generated by the HTML
	 * clients. Each HTML client can consist of serveral (or none) sub-clients
	 * that are responsible for rendering certain sub-parts of the output. The
	 * sub-clients can contain HTML clients themselves and therefore a
	 * hierarchical tree of HTML clients is composed. Each HTML client creates
	 * the output that is placed inside the container of its parent.
	 *
	 * At first, always the HTML code generated by the parent is printed, then
	 * the HTML code of its sub-clients. The order of the HTML sub-clients
	 * determines the order of the output of these sub-clients inside the parent
	 * container. If the configured list of clients is
	 *
	 *  array( "subclient1", "subclient2" )
	 *
	 * you can easily change the order of the output by reordering the subparts:
	 *
	 *  client/html/<clients>/subparts = array( "subclient1", "subclient2" )
	 *
	 * You can also remove one or more parts if they shouldn't be rendered:
	 *
	 *  client/html/<clients>/subparts = array( "subclient1" )
	 *
	 * As the clients only generates structural HTML, the layout defined via CSS
	 * should support adding, removing or reordering content by a fluid like
	 * design.
	 *
	 * @param array List of sub-client names
	 * @since 2014.03
	 * @category Developer
	 */
	private $_subPartPath = 'client/html/catalog/list/default/subparts';

	/** client/html/catalog/list/head/name
	 * Name of the head part used by the catalog list client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Catalog_List_Head_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2014.03
	 * @category Developer
	 */

	/** client/html/catalog/list/promo/name
	 * Name of the promotion part used by the catalog list client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Catalog_List_Promo_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2014.03
	 * @category Developer
	 */

	/** client/html/catalog/list/pagination/name
	 * Name of the pagination part used by the catalog list client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Catalog_List_Pagination_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2014.03
	 * @category Developer
	 */

	/** client/html/catalog/list/items/name
	 * Name of the items part used by the catalog list client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Catalog_List_Items_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2014.03
	 * @category Developer
	 */
	private $_subPartNames = array( 'head', 'promo', 'pagination', 'items', 'pagination' );

	private $_cache;


	/**
	 * Returns the HTML code for insertion into the body.
	 *
	 * @return string HTML code
	 */
	public function getBody()
	{
		try
		{
			$view = $this->_setViewParams( $this->getView() );

			$html = '';
			foreach( $this->_getSubClients( $this->_subPartPath, $this->_subPartNames ) as $subclient ) {
				$html .= $subclient->setView( $view )->getBody();
			}
			$view->listBody = $html;
		}
		catch( Client_Html_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'client/html', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( Controller_Frontend_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'controller/frontend', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( MShop_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'mshop', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( Exception $e )
		{
			$context = $this->_getContext();
			$context->getLogger()->log( $e->getMessage() . PHP_EOL . $e->getTraceAsString() );

			$view = $this->getView();
			$error = array( $context->getI18n()->dt( 'client/html', 'A non-recoverable error occured' ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}

		/** client/html/catalog/list/default/template-body
		 * Relative path to the HTML body template of the catalog list client.
		 *
		 * The template file contains the HTML code and processing instructions
		 * to generate the result shown in the body of the frontend. The
		 * configuration string is the path to the template file relative
		 * to the layouts directory (usually in client/html/layouts).
		 *
		 * You can overwrite the template file configuration in extensions and
		 * provide alternative templates. These alternative templates should be
		 * named like the default one but with the string "default" replaced by
		 * an unique name. You may use the name of your project for this. If
		 * you've implemented an alternative client class as well, "default"
		 * should be replaced by the name of the new class.
		 *
		 * @param string Relative path to the template creating code for the HTML page body
		 * @since 2014.03
		 * @category Developer
		 * @see client/html/catalog/list/default/template-header
		 */
		$tplconf = 'client/html/catalog/list/default/template-body';
		$default = 'catalog/list/body-default.html';

		return $view->render( $this->_getTemplate( $tplconf, $default ) );
	}


	/**
	 * Returns the HTML string for insertion into the header.
	 *
	 * @return string String including HTML tags for the header
	 */
	public function getHeader()
	{
		try
		{
			$view = $this->_setViewParams( $this->getView() );

			$html = '';
			foreach( $this->_getSubClients( $this->_subPartPath, $this->_subPartNames ) as $subclient ) {
				$html .= $subclient->setView( $view )->getHeader();
			}
			$view->listHeader = $html;
		}
		catch( Exception $e )
		{
			$this->_getContext()->getLogger()->log( $e->getMessage() . PHP_EOL . $e->getTraceAsString() );
			return '';
		}

		/** client/html/catalog/list/default/template-header
		 * Relative path to the HTML header template of the catalog list client.
		 *
		 * The template file contains the HTML code and processing instructions
		 * to generate the HTML code that is inserted into the HTML page header
		 * of the rendered page in the frontend. The configuration string is the
		 * path to the template file relative to the layouts directory (usually
		 * in client/html/layouts).
		 *
		 * You can overwrite the template file configuration in extensions and
		 * provide alternative templates. These alternative templates should be
		 * named like the default one but with the string "default" replaced by
		 * an unique name. You may use the name of your project for this. If
		 * you've implemented an alternative client class as well, "default"
		 * should be replaced by the name of the new class.
		 *
		 * @param string Relative path to the template creating code for the HTML page head
		 * @since 2014.03
		 * @category Developer
		 * @see client/html/catalog/list/default/template-body
		 */
		$tplconf = 'client/html/catalog/list/default/template-header';
		$default = 'catalog/list/header-default.html';

		return $view->render( $this->_getTemplate( $tplconf, $default ) );
	}


	/**
	 * Returns the sub-client given by its name.
	 *
	 * @param string $type Name of the client type
	 * @param string|null $name Name of the sub-client (Default if null)
	 * @return Client_Html_Interface Sub-client object
	 */
	public function getSubClient( $type, $name = null )
	{
		return $this->_createSubClient( 'catalog/list/' . $type, $name );
	}


	/**
	 * Processes the input, e.g. store given values.
	 * A view must be available and this method doesn't generate any output
	 * besides setting view variables.
	 */
	public function process()
	{
		try
		{
			$this->_process( $this->_subPartPath, $this->_subPartNames );
		}
		catch( Client_Html_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'client/html', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( Controller_Frontend_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'controller/frontend', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( MShop_Exception $e )
		{
			$view = $this->getView();
			$error = array( $this->_getContext()->getI18n()->dt( 'mshop', $e->getMessage() ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
		catch( Exception $e )
		{
			$context = $this->_getContext();
			$context->getLogger()->log( $e->getMessage() . PHP_EOL . $e->getTraceAsString() );

			$view = $this->getView();
			$error = array( $context->getI18n()->dt( 'client/html', 'A non-recoverable error occured' ) );
			$view->listErrorList = $view->get( 'listErrorList', array() ) + $error;
		}
	}


	/**
	 * Sets the necessary parameter values in the view.
	 *
	 * @param MW_View_Interface $view The view object which generates the HTML output
	 * @return MW_View_Interface Modified view object
	 */
	protected function _setViewParams( MW_View_Interface $view )
	{
		if( !isset( $this->_cache ) )
		{
			$context = $this->_getContext();
			$config = $context->getConfig();

			$products = $this->_getProductList( $view );

			$text = (string) $view->param( 'f-search-text' );
			$catid = (string) $view->param( 'f-catalog-id' );

			if( $catid == '' ) {
				$catid = $config->get( 'client/html/catalog/list/catid-default', '' );
			}

			if( $text === '' && $catid !== '' )
			{
				$domains = $config->get( 'client/html/catalog/domains', array( 'media', 'text' ) );

				$catalogManager = MShop_Factory::createManager( $context, 'catalog' );
				$view->listCatPath = $catalogManager->getPath( $catid, $domains );

				$listCatPath = $view->get( 'listCatPath', array() );

				if( ( $categoryItem = end( $listCatPath ) ) !== false ) {
					$view->listCurrentCatItem = $categoryItem;
				}
			}

			/** client/html/catalog/list/stock/enable
			 * Enables or disables displaying product stock levels in product list views
			 *
			 * This configuration option allows shop owners to display product
			 * stock levels for each product in list views or to disable
			 * fetching product stock information.
			 *
			 * The stock information is fetched via AJAX and inserted via Javascript.
			 * This allows to cache product items by leaving out such highly
			 * dynamic content like stock levels which changes with each order.
			 *
			 * @param boolean Value of "1" to display stock levels, "0" to disable displaying them
			 * @since 2014.03
			 * @category User
			 * @category Developer
			 * @see client/html/catalog/detail/stock/enable
			 * @see client/html/catalog/stock/url/target
			 * @see client/html/catalog/stock/url/controller
			 * @see client/html/catalog/stock/url/action
			 * @see client/html/catalog/stock/url/config
			 */
			if( !empty( $products ) && $config->get( 'client/html/catalog/list/stock/enable', true ) === true )
			{
				/** client/html/catalog/stock/url/target
				 * Destination of the URL where the controller specified in the URL is known
				 *
				 * The destination can be a page ID like in a content management system or the
				 * module of a software development framework. This "target" must contain or know
				 * the controller that should be called by the generated URL.
				 *
				 * @param string Destination of the URL
				 * @since 2014.03
				 * @category Developer
				 * @see client/html/catalog/stock/url/controller
				 * @see client/html/catalog/stock/url/action
				 * @see client/html/catalog/stock/url/config
				 */
				$stockTarget = $config->get( 'client/html/catalog/stock/url/target' );

				/** client/html/catalog/stock/url/controller
				 * Name of the controller whose action should be called
				 *
				 * In Model-View-Controller (MVC) applications, the controller contains the methods
				 * that create parts of the output displayed in the generated HTML page. Controller
				 * names are usually alpha-numeric.
				 *
				 * @param string Name of the controller
				 * @since 2014.03
				 * @category Developer
				 * @see client/html/catalog/stock/url/target
				 * @see client/html/catalog/stock/url/action
				 * @see client/html/catalog/stock/url/config
				 */
				$stockController = $config->get( 'client/html/catalog/stock/url/controller', 'catalog' );

				/** client/html/catalog/stock/url/action
				 * Name of the action that should create the output
				 *
				 * In Model-View-Controller (MVC) applications, actions are the methods of a
				 * controller that create parts of the output displayed in the generated HTML page.
				 * Action names are usually alpha-numeric.
				 *
				 * @param string Name of the action
				 * @since 2014.03
				 * @category Developer
				 * @see client/html/catalog/stock/url/target
				 * @see client/html/catalog/stock/url/controller
				 * @see client/html/catalog/stock/url/config
				 */
				$stockAction = $config->get( 'client/html/catalog/stock/url/action', 'stock' );

				/** client/html/catalog/stock/url/config
				 * Associative list of configuration options used for generating the URL
				 *
				 * You can specify additional options as key/value pairs used when generating
				 * the URLs, like
				 *
				 *  client/html/<clientname>/url/config = array( 'absoluteUri' => true )
				 *
				 * The available key/value pairs depend on the application that embeds the e-commerce
				 * framework. This is because the infrastructure of the application is used for
				 * generating the URLs. The full list of available config options is referenced
				 * in the "see also" section of this page.
				 *
				 * @param string Associative list of configuration options
				 * @since 2014.03
				 * @category Developer
				 * @see client/html/catalog/stock/url/target
				 * @see client/html/catalog/stock/url/controller
				 * @see client/html/catalog/stock/url/action
				 * @see client/html/url/config
				 */
				$stockConfig = $config->get( 'client/html/catalog/stock/url/config', array() );

				$productIds = array_keys( $products );
				sort( $productIds );

				$params = array( 's-product-id' => implode( ' ', $productIds ) );
				$view->listStockUrl = $view->url( $stockTarget, $stockController, $stockAction, $params, array(), $stockConfig );
			}

			$view->listParams = $this->_getClientParams( $view->param() );
			$view->listPageCurr = $this->_getProductListPage( $view );
			$view->listPageSize = $this->_getProductListSize( $view );
			$view->listProductTotal = $this->_getProductListTotal( $view );
			$view->listProductSort = $view->param( 'f-sort', 'relevance' );
			$view->listProductItems = $products;

			$this->_cache = $view;
		}

		return $this->_cache;
	}
}
